/*
 * NFTADeterminizer.cpp
 *
 *  Created on: 30. 3. 2015
 *	  Author: Stepan Plachy
 */

#include "common/NFACommon.h"

#include <automaton/TA/NFTA.h>
#include <deque>
#include <algorithm>

namespace automaton {

namespace determinize {

std::set<label::Label> getTransitionRightSide(const NFTA & nfta, const std::ranked_symbol < > & symbol, const std::vector<label::Label> & states) {
	std::set<label::Label> res;
	for (const auto & transition : nfta.getTransitions()) {
		if (transition.first.first != symbol) continue;

		unsigned i = symbol.getRank().getData();
		for (; i > 0; i--)
			if (!recreateNFAStates(states[i - 1]).count(transition.first.second[i - 1])) break;

		if (i == 0) res.insert(transition.second.begin(), transition.second.end());
	}
	return res;
}

DFTA < > Determinize::determinize(const NFTA & nfta) {
	DFTA < > res;
	res.setInputAlphabet(nfta.getInputAlphabet());
	for (const auto & state : nfta.getStates()) res.addState(createDFAState({state}));
	for (const auto & state : nfta.getFinalStates()) res.addFinalState(createDFAState({state}));

	std::deque<label::Label> todo;

	//std::map<std::pair<std::ranked_symbol < >, std::vector<label::Label> >, std::set<label::Label> > transitions;
	for (const auto & transition : nfta.getTransitions()) {
		label::Label state = createDFAState(transition.second);
		std::vector<label::Label> states;
		for (const auto & s : transition.first.second) states.push_back(createDFAState({s}));
		if (transition.second.size() > 1 && res.addState(state)) todo.push_back(state);
		res.addTransition(transition.first.first, states, state);
	}
	
	while(!todo.empty()) {
		label::Label currentState = todo.front();
		todo.pop_front();
		std::set<label::Label> nftaStates = recreateNFAStates(currentState);

		std::deque<std::pair<std::pair<std::ranked_symbol < >, std::vector<label::Label> >, label::Label> > transitions;
		std::deque<unsigned> stops;
		for (const auto & transition : res.getTransitions()) {
			transitions.push_back(transition);
			stops.push_back(0);
		}

		while(!transitions.empty()) {
			auto transition = transitions.front();
			unsigned stop = stops.front();
			transitions.pop_front();
			stops.pop_front();
			const std::vector<label::Label> & states = transition.first.second;

			for (unsigned i = stop; i < states.size(); i++) {
				if (recreateNFAStates(states[i]).size() != 1) continue;
				label::Label nftaState = *recreateNFAStates(states[i]).begin();
				if (nftaStates.count(nftaState)) {
					std::vector<label::Label> newStates = states;
					newStates[i] = currentState;
					
					std::set<label::Label> newNextStates = getTransitionRightSide(nfta, transition.first.first, newStates);
					if (!newNextStates.empty()) {
						label::Label newNextState = createDFAState(newNextStates);
						if (res.addState(newNextState)) todo.push_back(newNextState);
						if (res.addTransition(transition.first.first, newStates, newNextState) && i + 1!= states.size()) {
							transitions.push_back(std::make_pair(std::make_pair(transition.first.first, newStates), newNextState));
							stops.push_back(i + 1);
						}
					}
				}
			}
		}

		for (const auto & state : nftaStates) {
			if (nfta.getFinalStates().count(state)) {res.addFinalState(currentState); break;}
		}

	}

	return res;
}

auto DeterminizeNFTA = Determinize::RegistratorWrapper<automaton::DFTA < >, automaton::NFTA>( Determinize::determinize );

} /* namespace determinize */

} /* namespace automaton */
