/*
 * Accept.h
 *
 *  Created on: 9. 2. 2014
 *      Author: Jan Travnicek
 */

#ifndef _AUTOMATON_ACCEPT_H__
#define _AUTOMATON_ACCEPT_H__

#include <core/multipleDispatch.hpp>
#include <automaton/Automaton.h>
#include <automaton/AutomatonFeatures.h>
#include <object/Object.h>
#include <string/LinearString.h>
#include <tree/ranked/RankedTree.h>

namespace automaton {

namespace run {

class Accept : public std::DoubleDispatch < Accept, bool, const automaton::AutomatonBase &, const alib::ObjectBase & > {
public:
	/**
	 * Performs conversion.
	 * @return left regular grammar equivalent to source automaton.
	 */
	static bool accept ( const automaton::Automaton & automaton, const alib::Object & object );
	static bool accept ( const automaton::Automaton & automaton, const string::LinearString < > & string );
	static bool accept ( const automaton::Automaton & automaton, const tree::RankedTree < > & string );

	static bool accept ( const automaton::DFA<> & automaton, const string::LinearString < > & string );
	static bool accept ( const automaton::NFA < > & automaton, const string::LinearString < > & string );
	static bool accept ( const automaton::DFTA < > & automaton, const tree::RankedTree < > & tree );
	static bool accept ( const automaton::NFTA & automaton, const tree::RankedTree < > & tree );
	static bool accept ( const automaton::InputDrivenDPDA < > & automaton, const string::LinearString < > & string );
	static bool accept ( const automaton::VisiblyPushdownDPDA < > & automaton, const string::LinearString < > & string );
	static bool accept ( const automaton::RealTimeHeightDeterministicDPDA < > & automaton, const string::LinearString < > & string );
	static bool accept ( const automaton::DPDA < > & automaton, const string::LinearString < > & string );
	static bool accept ( const automaton::NPDTA < > & automaton, const string::LinearString < > & string );

};

} /* namespace run */

} /* namespace automaton */

#endif /* _AUTOMATON_ACCEPT_H__ */
